var Class = require('../class/Class.js').Class;
var assertions = require('../common/Assertions.js');
var types = require('../common/Types.js');
var net = require('net');
var ConnectionPool = require('../net/ConnectionPool.js').ConnectionPool;



var Memcache = function(){
	
	this._lifetime = 10;
    this._connectionPool = null;

	this.initialize = function(pool){
		this._connectionPool = pool;
	};
    
	this.get = function(key, callback){
        //sprawdzam w this._memory czy jest key
        //jesli nie to dalej
        //jesli jest to wisimy czyli callback 
        assertions.isString(key, Memcache.Exception.WRONG_TYPE);
        
        
        var that = this;
        
        this._connectionPool.getSocket(function(socket){
            
            var resBuff = new Buffer(0);
            
            socket.on('data', function(e){

                var tmp = Buffer(resBuff.length + e.length);
                resBuff.copy(tmp);
                e.copy(tmp, resBuff.length);
                resBuff = tmp;
                
                
                // jezeli E(alue) lub C(ommand bad format)
                if(resBuff[0]==69 || resBuff[0]==67){
                    that._connectionPool.givebackSocket(socket);
                    callback(null);
                    return;
                }
                
                if(resBuff.slice(resBuff.length - 5).toString('ascii')==='END\r\n'){
                    
                    that._connectionPool.givebackSocket(socket);
                    
                    var dataStart = 0;
                    for(var i=0;i<resBuff.length;i++){
                        if(resBuff[i]===13 && resBuff[i+1]===10){
                            dataStart = i+2;
                            break;
                        }
                    }
                    callback(resBuff.slice(dataStart, resBuff.length-7));
                }                               
                
            });
            
            var resultBuff = new Buffer('get ' + key +'\r\n', 'binary');
            socket.write(resultBuff);
        });
        
	};

	this.set = function(key, value, callback, lifeTime){
        
        lifeTime = lifeTime || this._lifetime;
        
        var that = this;
        
        //TODO dac zeby bylo reply i wtedy oddawac socket
		assertions.isString(key, Memcache.Exception.WRONG_TYPE);

        this._connectionPool.getSocket(function(socket){
            if(!Buffer.isBuffer(value)){
                value = new Buffer(value, 'binary');
            }
            
            var cmdBuffStart = new Buffer('set ' + key + ' 0 ' + lifeTime + ' ' + value.length + ' noreply\r\n', 'binary');
            var cmdBuffEnd = new Buffer('\r\n', 'binary');
            var resultBuff = new Buffer(cmdBuffStart.length + value.length + cmdBuffEnd.length);
            cmdBuffStart.copy(resultBuff);
            value.copy(resultBuff, cmdBuffStart.length);
            cmdBuffEnd.copy(resultBuff, (cmdBuffStart.length + value.length));
            socket.write(resultBuff, function(){
                that._connectionPool.givebackSocket(socket);
            });
        });
	};
    
    this['delete'] = function(key, callback){
        var that = this;
        
        //TODO dac zeby bylo reply i wtedy oddawac socket
		assertions.isString(key, Memcache.Exception.WRONG_TYPE);
        
        this._connectionPool.getSocket(function(socket){
            var resultBuff = new Buffer('delete ' + key + ' noreply\r\n', 'binary');
            socket.write(resultBuff, function(){
                that._connectionPool.givebackSocket(socket);
            });
        });
        
    };

	this.setLifetime = function(lifetime){
		assertions.isNumber(lifetime, Memcache.Exception.WRONG_LIFETIME_TYPE);
		this._lifetime = lifetime;
		return this;
	};

	this.getLifetime = function(){
		return this._lifetime;
	};
};
Memcache = new Class(new Memcache());
Memcache.Exception = {};
Memcache.Exception.WRONG_KEY_TYPE = "Key has to be a string";
Memcache.Exception.WRONG_VALUE_TYPE = "Value has to be a string, number or object";
Memcache.Exception.WRONG_LIFETIME_TYPE = "Lifetime has to be a number";
exports.Memcache = Memcache;
