var Class = require('../class/Class.js').Class;
var Assertions = require('../common/Assertions.js');

var EventDispatcher = require('../event/EventDispatcher.js').EventDispatcher;
var Event = require('../event/Event.js').Event;
var ErrorEvent = require('../event/ErrorEvent.js').ErrorEvent;

var OpalRequest = require('../opal/OpalRequest.js').OpalRequest;
var OpalLoader = require('../opal/OpalLoader.js').OpalLoader;
var Loader = require('../net/Loader.js').Loader;

var JsonRpcResponse = require('../jsonrpc/JsonRpcResponse.js').JsonRpcResponse;

var SingletonEnforcer = function(){};
SingletonEnforcer = new Class(new SingletonEnforcer());

var CredentialsManagerInstance = null;

var CredentialsManager = function(){
	this.Extends = EventDispatcher;
	this._credentials = {};

	this.initialize = function(enforcer){
		Assertions.isInstanceOf(enforcer, SingletonEnforcer, CredentialsManager.Exception.SINGLETON);
	};

	this.addEventListener = function(eventName, key, callback){
		return this.parent(eventName + key, callback);
	};

	this.getCredential = function(key){
		if(this._credentials.hasOwnProperty(key)){
			this.dispatchEvent(new Event(CredentialsManager.Event.LOADED + key, this._credentials[key]));
		}else{
			this.renewCredential(key);
		}
	};

	this.renewCredential = function(key){
		var self = this;

		(new OpalLoader(new OpalRequest({
			"url": "router.opaladmin.onetapi.pl",
			"method": "get_credentials",
			"params": {
				"ds_alias": key
			},
			"application": "edgeserver.edgeserver.onetapi.pl"
		}))).addEventListener(OpalLoader.Event.JSON_RESPONSE, function(data){
			var rpc = data.data.getBody();
            
			if(rpc.isError()){
				self.dispatchEvent(new ErrorEvent(CredentialsManager.Event.ERROR + key, rpc.getError()));
				return;
			}

			self._credentials[key] = rpc.getResult();
			self.dispatchEvent(new Event(CredentialsManager.Event.LOADED + key, self._credentials[key]));
		}).addEventListener(Loader.Event.ERROR, function(data){
			self.dispatchEvent(new ErrorEvent(CredentialsManager.Event.ERROR + key, data));
		}).load();
	};
};

CredentialsManager = new Class(new CredentialsManager());

CredentialsManager.getInstance = function(){
	if(!CredentialsManagerInstance){
		CredentialsManagerInstance = new CredentialsManager(new SingletonEnforcer());
	}

	return CredentialsManagerInstance;
};

CredentialsManager.Event = {};
CredentialsManager.Event.LOADED = 'CredentialsManager_LOADED_';
CredentialsManager.Event.ERROR = 'CredentialsManager_ERROR_';

CredentialsManager.Exception = {};
CredentialsManager.Exception.SINGLETON = "Singleton class, use getInstance() instead.";

exports.CredentialsManager = CredentialsManager;
