var Class = require("../class/Class.js").Class;
var Types = require("../common/Types.js");
var Iconv = require('iconv').Iconv;
var Base64 = require('base64');

var Data = function(){
    this.encoding = null;
    this.characterEncoding = null;
    this.data = null;
    this.initialize = function(data, encoding, characterEncoding){
        this.encoding = encoding;
        this.characterEncoding = characterEncoding;
        this.data = data;        
    }
    this.convertTo = function(encoding, characterEncoding){
        if(this.encoding != encoding || this.characterEncoding != characterEncoding){
            //jesli aktualnie mamy base64 musimy to odczynic na chwile
            if(this.encoding == BinaryData.Encoding.BASE64){
                this.data = new Buffer(Base64.decode(this.data), 'binary');
                this.encoding = BinaryData.Encoding.BINARY;
            }
            //jesli zmieniamy kodowanie znaków
            if(characterEncoding && this.characterEncoding != characterEncoding){
                this.data = (new Iconv(this.characterEncoding, characterEncoding)).convert(this.data);
                this.characterEncoding = characterEncoding;
                this.encoding = BinaryData.Encoding.BINARY;
            }
            //ustalamy ostateczny encoding
            if(this.encoding != encoding){
                //jesli ma byc base64
                if(encoding == BinaryData.Encoding.BASE64){
                    this.data = Base64.encode(this.data);
                    this.encoding = BinaryData.Encoding.BASE64;
                }
                //jesli powinien byc TEXT
                else if(encoding == BinaryData.Encoding.TEXT){
                    //tylko jesli kodowanie jest UTF8
                    if( this.characterEncoding == BinaryData.CharacterEncoding.UTF8 ){
                        this.encoding = BinaryData.Encoding.TEXT;
                        //this.data = this.data.toString();
                    }
                    //jesli nie jest to mamy problem
                    else{
                        throw BinaryData.Exception.CONVERTION_ERROR;
                    }
                }
            }
        }
    };
};
Data = new Class( new Data() );

var BinaryData = function(){
    this._encoding = null;
    this._characterEncoding = null;
    this._data = null;
    this.initialize = function(data, encoding, characterEncoding){
        this._encoding = encoding;
        this._characterEncoding = characterEncoding;
        this.setData(data, encoding, characterEncoding);
    };
    this.setData = function(data, encoding, characterEncoding){
        this._data = new Data(data, encoding, characterEncoding);
    };
    this.length = function(){
        if(this._data){
            //Potential problem maker if given buffer will have more space than content
            return this.toBuffer().length;
        }
    };
    this.getCharacterEncoding = function(){
        return this._characterEncoding;
    };
    this.getEncoding = function(){
        return this._encoding;
    };
    this.toUTF8String = function(){
        //TODO make not permanent convertion
        this._data.convertTo(BinaryData.Encoding.TEXT, BinaryData.CharacterEncoding.UTF8);
        return this._data.data.toString();   
    }
    this.toString = function(){
        var encoding = BinaryData.Encoding.TEXT;
        if(this.getEncoding() != BinaryData.Encoding.TEXT){
            encoding = BinaryData.Encoding.BASE64;
        }
        this._data.convertTo(encoding, this.getCharacterEncoding())
        return this._data.data.toString();
    };
    this.toBuffer = function(){
        this._data.convertTo(this.getEncoding(), this.getCharacterEncoding());
        if(Buffer.isBuffer(this._data.data)){
            return this._data.data;
        }
        else{
            return new Buffer(this._data.data, this.getEncoding());
        }
    };
};
BinaryData = new Class( new BinaryData() );


BinaryData.Exception = {};
BinaryData.Exception.CONVERTION_ERROR = "CONVERTION_ERROR";

BinaryData.Encoding = {};
BinaryData.Encoding.BINARY = "binary";
BinaryData.Encoding.BASE64 = "base64";
BinaryData.Encoding.TEXT   = "utf8";
// BinaryData.Encoding.ASCII  = "ascii";
// BinaryData.Encoding.UCS2   = "ucs2";
// BinaryData.Encoding.HEX   = "hex"; 

BinaryData.CharacterEncoding = {};
BinaryData.CharacterEncoding.UTF8 = "UTF-8";
BinaryData.CharacterEncoding.ISO_8859_1  = "ISO-8859-1";
BinaryData.CharacterEncoding.ISO_8859_2  = "ISO-8859-2";

exports.BinaryData = BinaryData;    