var Event = require('./Event.js').Event;
var Class = require("../class/Class.js").Class;

/**
 * @class ErrorEvent
 * @extends Event
 * @requires Class
 * @requires Event
 *
 * @param {String} type
 * @param {Object} data
 * @param {Number} code
 * @param {String} message
 */
var ErrorEvent = function() {
    
    /** @ignore */
	this.Extends = Event;

    /**
     * Error code.
     * @public
     * @type {Number}
     */
	this.code = 0;

    /**
     * Error message.
     * @public
     * @type {String} 
     */
	this.message = null;


    /** @ignore */
	this.initialize = function(type, data, code, message) {
	    if(data instanceof ErrorEvent){
	        data = data.toJson();
	    }
	    /*else if (data instanceof Object){
	        data = JSON.stringify(data);
	    }*/
		this.parent(type, data);
		this.code = code;
		this.message = message;
	};

    this.toJson = function(){
        var tmpObj = {
            code : this.code,
            type : this.type,
            data : this.data,
            message :  this.message
        };
        return tmpObj;
    }
    /**
     * Displays error as html content
     * @return {String}
     */
    this.toHtmlString = function(){
        var retVal = "<pre>";
        for(el in this){
            if(this.hasOwnProperty(el) && typeof this[el]!='function' ){
                retVal+="\t" + el+': '+this[el]+"\n";
            }
        }
        retVal += "</pre>";
        return retVal;
    };
    
};

ErrorEvent = new Class(new ErrorEvent());
exports.ErrorEvent = ErrorEvent;
