var Class = require("../class/Class.js").Class;
var JsonRpcAbstract = require("./JsonRpcAbstract.js").JsonRpcAbstract;
var JsonRpcError = require('./JsonRpcError.js').JsonRpcError;

/**
 * @class JsonRpcResult
 * @extends JsonRpcAbstract
 * @requires Class
 * @requires JsonRpcAbstract
 *
 * @param {Object} result
 */
var JsonRpcResponse = function(){

    /** @ignore */
    this.Extends = JsonRpcAbstract;
    
    /**
     * result
     * 
     * @private
     * @property {Object} _result
     */
   
    this._id = null;
    this._result = null;
    this._error = null;
    
    /** @ignore */
    this.initialize = function(/*data|id, result, error*/){
		this.parent.apply(this, arguments);
    };

	this._init = function(args){
		if(args[0]){
			this.setId(args[0]);
		}else{
			throw JsonRpcAbstract.Exceptions.INVALID_JSON_RPC;
		}

		if(args[1]){
			this.setResult(args[1]);
		}

		if(args[2]){
			this.setError(new JsonRpcError(args[2]));
		}
	}

	this._parse = function(json){
		return [
			json.id,
			json.result,
			json.error
		];
	};

    this.setId = function(id){
        if(typeof id !== 'string'){
            throw "Id should be a String";
        }

        this._id = id;

		return this;
    };

	this.getId = function(){
		return this._id;
	}
    
    /**
     * sets result
     * 
     * @param {Object} result
     * @returns {JsonRpcResult}
     */
    this.setResult = function(result){
        this._result = result;

        return this;
    };

	/**
     * gets result
     * 
     * @returns {Object}
     */
	this.getResult = function(){
		return this._result;
	};

    this.setError = function(code, message, data){
        if(code instanceof JsonRpcError ){
            this._error = code;
        } else {
            this._error = new JsonRpcError(code, message, data);
        }

		return this;
    };

	this.getError = function(){
		return this._error;
	};
    
    this.isError = function(){
        return this._error !== null;
    }
    
    /**
     * returns properties as json
     * 
     * @returns {Object}
     */
    this.toJson = function(){
        var jsonObj = {
            jsonrpc: this._jsonrpc,
            id: this._id,
            result: this._result
        };

        if(this._error){
            jsonObj.error = this._error.toJson();
        }

        return jsonObj;
    };
};

JsonRpcResponse = new Class(new JsonRpcResponse());

exports.JsonRpcResponse = JsonRpcResponse;
