var Core = require('core');

var CredentialConnectionPool = function(){
	this.Extends = Core.Net.ConnectionPool;

	this._cm = null;
	this._throttleDisconnects = false;
	this._credentialKey;
	this._concurrentDisconnects = 0;
	this._maxConcurrentDisconnects = 50;
	this._concurrentDisconnectsRatio = 0.1;

	this.initialize = function(credentialKey){
		var self = this;

		this._credentialKey = credentialKey;

		this._cm = Core.Credentials.CredentialsManager.getInstance();
		this._cm.addEventListener(Core.Credentials.CredentialsManager.Event.LOADED, credentialKey, function(data){
			self._options = {
				host: data.data.hosts[0].host,
				port: data.data.hosts[0].port,
				connections: data.data.connectionPool
			};
            
			console.log('    - CredentialConnectionPool: Received Credentials', self._options);

			if(self._sockets.length == 0){
				self._connect();
			}else{
				self._reconnect();
			}
		}).addEventListener(Core.Credentials.CredentialsManager.Event.ERROR, credentialKey, function(data){
			console.log('    - CredentialConnectionPool: Error while receiving credential data', data);
			setTimeout(function(){
				console.log('    - CredentialConnectionPool: Retrying...');
				self._cm.getCredential(credentialKey);
			}, 100);
		});
		this._cm.getCredential(credentialKey);
	};

	this._connect = function(){
		console.log('    - CredentialConnectionPool: Connecting');
		this.parent();
	};

	this._reconnect = function(){
		console.log('    - CredentialConnectionPool: Reconnecting');
		for(var i = 0, l = this._sockets.length; i < l; i++){
			//console.log('Destroying and reconnecting to socket:', i, this._options);
			this._sockets[i]._socket.destroy();
			this._sockets[i]._socket.connect(this._options.port, this._options.host);
		}

		this._throttleDisconnects = false;
	};

	this._refineSocketHandlers = function(socket){
		this.parent(socket);

		var self = this;

		socket.on('connect', function(){
			socket._disconnects = 0;
			self._concurrentDisconnects--;
		});
		if(socket.__onClose == undefined){
			socket.__onClose = socket._onClose;
		}

		socket._onClose = function(sck){
			(socket._disconnects) ? socket._disconnects++ : socket._disconnects = 1;
			self._concurrentDisconnects++;

			if(!self._throttleDisconnects){
				console.log('    - CredentialConnectionPool: Disconnects: ' + socket._disconnects + ' | Concurrent Disconnects: ' + self._concurrentDisconnects);
				if(socket._disconnects >= 5 || self._concurrentDisconnects >= self._maxConcurrentDisconnects){
					self._throttleDisconnects = true;
					
					setTimeout(function(){
						self._cm.getCredential(self._credentialKey);
					}, 100);
				}else{
					socket.__onClose(sck);
				}
			}
		};
	};
};

CredentialConnectionPool = new Core.Class(new CredentialConnectionPool());
exports.CredentialConnectionPool = CredentialConnectionPool;
