var Class = require("../class/Class.js").Class;
var Request = require("./Request.js").Request;
var Response = require("./Response.js").Response;
var BinaryData = require("./../data/BinaryData.js").BinaryData;
var RequestProcessor = require("./RequestProcessor.js").RequestProcessor;
var http = require('http');

/**
 * @class Server Main http server class. Wraps native node.js http object. Creates Request processor instance and dispatch processing. 
 * @param {Class} RequestProcessorClass Subclass of RequestProcessor family.
 */
var Server = function(){
    this.RequestProcessorClass = null;
    this._server = null;
    this.initialize = function(RequestProcessorClass){
        this.RequestProcessorClass = RequestProcessorClass;
        var that = this;
        
        this._server = http.createServer(function(request, response) {
            that._onRequestHandler(request, response);
        });
    };
    
    this.listen = function(port, host){
        this._server.listen(port, host);
    };
    
    this.listenFD = function(fd, protocol){
        this._server.listenFD(fd, protocol);  
    };
    /**
     * Returns native http server instance.
     * @public
     * @returns http.Server
     */
    this.getClusterNode = function(){
        return this._server;
    };
    /**
     * @private
     */
    this._onRequestHandler = function(httpRequest, httpResponse) {
        
        console.log('\n-- request started');

        var that = this;
        var postData = [];
        httpRequest.on('end', function() {
            httpRequest.data = new BinaryData(
                postData.join(""), 
                BinaryData.Encoding.TEXT, 
                BinaryData.CharacterEncoding.UTF8
            );
            var request = new Request(httpRequest);
            var response = new Response(httpResponse);
            var requestProcessor = that.createRequestProcessor();
            requestProcessor.process(request, response);
        });
        httpRequest.on('data', function(data) {
            postData.push(data);
        });
    };
    
    this.createRequestProcessor = function(){
        var requestProcessor = new this.RequestProcessorClass(this);
        if(!(requestProcessor instanceof  RequestProcessor)){
            throw Server.Exception.WRONG_PROCESSOR_CLASS;
        }
        return requestProcessor;
    };
}

Server = new Class(new Server());

/**
 * Namespace for exceptions messages.
 * @static
 * @namespace
 */
Server.Exception = {};
/**
 * @static
 * @public
 */
Server.Exception.WRONG_PROCESSOR_CLASS = "Request processor class must be inheriting RequestProcessor";
//

exports.Server = Server;
