var Class = require('../class/Class.js').Class;

var Net = require('net');

var Socket = function(){
	this._socket;
	this._data = [];
	this._listeners = ['connect', 'data', 'end', 'timeout', 'drain', 'error', 'close'];
	this._hasReconnected = false;

	this._port;
	this._host;

	this.initialize = function(port, host){
		this._port = port;
		this._host = host;
		this._socket = Net.createConnection(port, host);
	};

	this.on = this.addEventListener = function(eventName, eventCallback){
		return this._socket.on(eventName, eventCallback);
	};

	this.write = function(data, encoding, callback){
		var self = this;
		var index = this._data.push({data: data, encoding: encoding, callback: callback}) - 1;

		this._socket.write(data, encoding, function(){
			self._data[index] = undefined;
			callback();
		});
	};

	this.bufferSize = function(){
		return this._socket.bufferSize;
	};

	this._executeWrites = function(){
		for(var i = 0, l = this._data.length; i < l; i++){
			if(this._data[i] !== undefined){
				this.write(this._data[i].data, this._data[i].encoding, this._data[i].callback);
			}
		}
	};

	this._refineHandlers = function(){
		var self = this;

		this._listeners.push('sClose');

		for(var i = 0, l = this._listeners.length; i < l; i++){
			this._socket.removeAllListeners(this._listeners[i]);
		}

		this._hasReconnected = false;
		
		this._socket.on('connect', function(){
			if(self._hasReconnected){
                console.log('      - Socket: reconnected! ' + self._host + ':' + self._port);
				self._executeWrites();
			} else {
                console.log('      - Socket: connected! ' + self._host + ':' + self._port);
            }
		});
		this._socket.on('close', function(byError){
            if(byError){
                console.log('      - Socket: closed! (on error)');
            } else {
                console.log('      - Socket: closed! (NOT on error)');
            }
			self._onClose(this);
		});

		this._socket.on('error', function(error){
            console.log('      - Socket: error! -> ' + error.message);
		});
	};

	this._onClose = function(socket){
        var that = this;
		setTimeout(function(){
            console.log('      - Socket: trying to reconnect socket: ' + that._host + ':' + that._port);
			that._hasReconnected = true;
			socket.connect(that._port, that._host);
		}, 100);
	};
};

Socket = new Class(new Socket());
exports.Socket = Socket;
