var http = require('http');
var OpalRequest = require('./OpalRequest.js').OpalRequest;
var OpalResponse = require('./OpalResponse.js').OpalResponse;
var Event = require('../event/Event.js').Event;
var ErrorEvent = require('../event/ErrorEvent.js').ErrorEvent;
var Class = require('../class/Class.js').Class;
var Loader = require('../net/Loader.js').Loader;
var Response = require('../net/Response.js').Response;
var JsonRpcResponse = require('../jsonrpc/JsonRpcResponse.js').JsonRpcResponse;

/**
 * @class OpalLoader
 * @extends EventDispatcher
 * @requires Class
 * @requires OpalRequest
 * @requires Event
 * @requires ErrorEvent
 * @requires EventDispatcher
 * @requires http
 *
 * @param {OpalRequest} request
 */
var OpalLoader = function(){

    /** @ignore */
    this.Extends = Loader;

	this._parseResponse = function(response){
		var isJson = response.headers.hasOwnProperty('content-type') && response.headers['content-type'].indexOf('application/json') !== -1, data, body;

        if(isJson){
			var json = this._buffer.toString();

	        try {
	            body = new JsonRpcResponse(JSON.parse(json));
	        } catch(err) {
	            this.dispatchEvent(new ErrorEvent(Loader.Event.ERROR, {json: json, error: err.message}, -4, 'JSON parse error'));
	            return false;
	        }
		}else if(response.statusCode!=200){
            this.dispatchEvent(new ErrorEvent(Loader.Event.ERROR, {}, -5, 'bad request, status code != 200'));
            return false;
		}else{
			body = new Response(this._buffer);
		}
        
		data = new OpalResponse({
			"statusCode": response.statusCode,
			"headers": response.headers,
			"httpVersion": response.httpVersion,
			"body": body
		});

		if(isJson){
			this.dispatchEvent(new Event(OpalLoader.Event.JSON_RESPONSE, data));
		}else{
			this.dispatchEvent(new Event(OpalLoader.Event.HTTP_RESPONSE, data));
		}

		//zwracamy "blad" aby Loader oszukac dispatch Loader.Event.LOADED
		return false;
	};
};

OpalLoader = new Class(new OpalLoader());

OpalLoader.Event = {};
OpalLoader.Event.JSON_RESPONSE = 'OpalLoader_JSON_RESPONSE';
OpalLoader.Event.HTTP_RESPONSE = 'OpalLoader_HTTP_RESPONSE';

exports.OpalLoader = OpalLoader;
