var Class = require('../class/Class.js').Class;
var Request = require('../net/Request.js').Request;
var JsonRpcRequest = require('../jsonrpc/JsonRpcRequest').JsonRpcRequest;
var BinaryData = require('../data/BinaryData.js').BinaryData;


/**
 * @class OpalRequest
 * @extends UrlRequest
 * @requires Class
 * @requires UrlRequest
 *
 */
var OpalRequest = function(){
    
    /** @ignore */
    this.Extends = Request;

    /**
     * @private
     * @property {String} _remoteMethod
     */
    this._remoteMethod = null;

    /** @ignore */
    this.initialize = function(data){
		if(data){
	        if(!data.hasOwnProperty('url') || !data.hasOwnProperty('params') || !data.hasOwnProperty('method')){
	            throw OpalRequest.Exception.INCOMPLETE_PARAMS;
	        }

	        if(data.url.substring(0, 8) == 'https://'){
		        this.setPort(443);
			    this.setHeader('host', data.url.replace('https://', '').replace('/', ''));
	        }else{
		        this.setPort(80);
			    this.setHeader('host', data.url.replace('http://', '').replace('/', ''));
	        }

			//this.setHost(data.url);
			this.setHost('rt0.int.dev.onet');

	        this.setRemoteMethod(data.method);
		    this.setMethod(Request.POST);

			if(data.hasOwnProperty('application')){
				this.setApplication(data.application);
	        }else{
		        this.setApplication('localhost.front.onetapi.pl');
			}

	        var data = new JsonRpcRequest({
			    'id': this.getRemoteMethod() + '-' + +new Date(),
				'method': this.getRemoteMethod(),
	            'params': data.params
		    }).toString(), bData = new BinaryData(data, BinaryData.Encoding.TEXT, BinaryData.CharacterEncoding.UTF8);

	        this.setBody(bData);

		    this.setHeaders({
			    'content-type' : 'application/json-rpc',
				'content-length' : bData.length()
	        });
		}
    };

    /**
     * sets the application ident string
     * @param {String} app
     * @returns {OpalRequest}
     */
    this.setApplication = function(app){
        this.setHeader('x-onet-app', app, true);

        return this;
    };

    /**
     * returns application ident string
     * @returns {OpalRequest}
     */
    this.getApplication = function(){
        return this.getHeader('x-onet-app');
    };


    /**
     * sets method of communicating with a service
     * @param {String} method
     * @returns {OpalRequest}
     */
    this.setRemoteMethod = function(method){
        this._remoteMethod = method;

        return this;
    };

    /**
     * returns method of communicating with a service
     * 
     * @returns {OpalRequest}
     */
    this.getRemoteMethod = function(){
        return this._remoteMethod;
    };
};

OpalRequest = new Class(new OpalRequest());

OpalRequest.Exception = {};
OpalRequest.Exception.INCOMPLETE_PARAMS = 'Incomplete params given';

exports.OpalRequest = OpalRequest;
